@@ -1,8 +1,8 @@
-## MMD.R (2018-01-08)
+## MMD.R (2022-05-12)
 
 ##   Mismatch Distribution
 
-## Copyright 2009-2017 Emmanuel Paradis, 2013 David Winter
+## Copyright 2009-2022 Emmanuel Paradis, 2013 David Winter
 
 ## This file is part of the R-package `pegas'.
 ## See the file ../DESCRIPTION for licensing issues.
@@ -11,8 +11,9 @@ MMD <- function(x, xlab = "Distance", main = "", rug = TRUE, legend = TRUE,
                 lcol = c("blue", "red"), lty = c(1, 1), bw = 2, ...)
 {
     d <- dist.dna(x, "N")
-    hist(d, xlab = xlab, main = main, freq = FALSE, ...)
-    lines(density(d, bw = bw), col = lcol[1], lty = lty[1])
+    h <- hist(d, xlab = xlab, main = main, freq = FALSE, ...)
+    dd <- density(d, bw = bw)
+    lines(dd, col = lcol[1], lty = lty[1])
     ## by David Winter:
     theta <- mean(d)
     upper <- ceiling(max(d))
@@ -29,4 +30,5 @@ MMD <- function(x, xlab = "Distance", main = "", rug = TRUE, legend = TRUE,
         #legend("topleft", c("Empirical", "Stable expectation"),
         #       lty = 1, col = lcol, bg = "white", bty = "n")
     }
+    invisible(list(histogram = h, empirical.density = dd, expected.curve = e))
 }
